// semi-implicit formulation of mass conservation (with Darcy's law -ie. ,
// momentum conservation- substituted inside the mass conservation equation)

// update explicit quantities
Beta  = eps_g * M / (R * Tg);
Gamma = p * M / (mu * R * Tg) * K;

// compute implicitly the new pressure
solve
(
    fvm::ddt(Beta, p)                                  // storage
    - fvm::div(fvc::interpolate(Beta)*mesh.phi(), p)   // mesh motion correction (ALE)
    - fvm::laplacian(Gamma, p)                         // convection
    - piTotal                                          // source : pyrolysis
    - omegaHeterogeneousTotal                          // source : heterogeneous reactions
);

// print min and max pressures in the terminal during the run
Info << "p_max = "
     << max(p).value()
     << " / p_min = "
     << min(p).value()
     << endl;

// average gas velocity (Darcy's law) from the new pressure gradient (cell centered)
v = - 1. / eps_g * 1. / mu * (K & fvc::grad(p));

// gas density (Ideal gas law)
rho_g = p * M / (R * Tg);

// pyrolysis gas mass flow rate (cell centered)
mDotG = eps_g * rho_g * v;

// pyrolysis gas mass flow rate (interpolated on the faces)
mDotGface =
    - fvc::interpolate(rho_g / mu) *
    fvc::interpolate(K) & mesh.Sf() / mesh.magSf() *
    fvc::snGrad(p);
gradP = fvc::grad(p);

// velocity
if (writeVelocityComponents)
{
    vi = v.component(0);
    vj = v.component(1);
    vk = v.component(2);
    vmag = mag(v);
}
// mean gas velocity
if (cfd)
{
    U = v * eps_g;
}
