// Model for a type A fibrous material - may be generalized (see Lachaud2010_JSR)

// update external radius density (matrix or fiber) for matrix-coated fibers being ablated
forAll (rho_ext, i)
{
    if (rT[i] <= rf0.value())
    {
        rho_ext[i] = rhoI[fiberPhase].value();
    }
    else if (matrixPhase > 0)
    {
        rho_ext[i] = rho[matrixPhase][i];
    }
    else
    {
        Info << "Error in the microscopic material model. Wrong initial fiber radius." 
             << "Add a matrix phase or increase radius." << nl;
        return (0);
    }
}

// Update solid mass removal by heterogenous reactions
m_dot_ablation = 0. * m_dot_ablation;

forAll(omegaHeterogeneousTotal, i)
{
    m_dot_ablation.value() +=
        omegaHeterogeneousTotal[i] * mesh.V()[i]; // total in-depth mass loss rate
}

// solid fraction evolution due to volume ablation
solve
(
      fvm::ddt(eps_s) - fvm::div(mesh.phi(), eps_s)  // radius evolution with ALE correction
    + omegaHeterogeneousTotal / rho_ext // mass loss rate
);

eps_s.max(0);

// update geometrical properties

rT = rT0 * sqrt(eps_s/eps_s_0);

specificSurface =  rT * 2 * eps_s_0 / (rT0*rT0);
specificSurface.max(0);

siteDensity =  siteDensityIni * (1. - exp(-pow4(rT / rff))); // smooth step function centered at rff (rT < rff -> 0, rT > rff -> 1)

// If we had a material model (conductivity, cp, etc) as a function of phase fractions we would update the fractions
// For now it is safer to not go into this level of detail when solving the energy conservation

if (energyConservation == "isothermal")
{
forAll(rT, i)
{
    if (rT[i] >= rf0.value())
    {
        if ((matrixPhase > 0))
        {
        eps[matrixPhase][i] =
            epsI[matrixPhase].value() *
            (rT[i] * rT[i] - rf0.value() * rf0.value()) /
            (rT0[i] * rT0[i]  - rf0.value() * rf0.value());
        }
        eps[fiberPhase][i] = epsI[fiberPhase].value();
    }
    else
    {
        if ((matrixPhase > 0))
        {
        eps[matrixPhase][i] = 0;
        }
        eps[fiberPhase][i] =
            epsI[fiberPhase].value() *
            (rT[i] * rT[i]) /
            (rf0.value() * rf0.value());
    }
}


scalarField rTint = rT.boundaryField()[movingPatchID].patchInternalField();
scalarField rT0int = rT0.boundaryField()[movingPatchID].patchInternalField();

forAll(rTint, i)
{
    if (rTint[i] >= rf0.value())
    {
        if ((matrixPhase > 0))
        {
        eps[matrixPhase].boundaryFieldRef()[movingPatchID][i] =
            epsI[matrixPhase].value() *
            (rTint[i] * rTint[i] - rf0.value() * rf0.value()) /
            (rT0int[i] * rT0int[i]  - rf0.value() * rf0.value());
        }
        eps[fiberPhase].boundaryFieldRef()[movingPatchID][i] = epsI[fiberPhase].value();
    }
    else
    {
        if ((matrixPhase > 0))
        {
        eps[matrixPhase].boundaryFieldRef()[movingPatchID][i] = 0. ;
        }
        eps[fiberPhase].boundaryFieldRef()[movingPatchID][i] =
            epsI[fiberPhase].value() *
            (rTint[i] * rTint[i]) /
            (rf0.value() * rf0.value()) ;
    }
}
} // end "isothermal"
 


