// This is to initialize the dynamic mesh motion -- identification of the moving
// patch (receding surface) and creation of the motion fields

Info << "Reading Motion Properties" << endl;

// Open the dictionary
IOdictionary dynamicMeshDict
(
    IOobject
    (
        "dynamicMeshDict",
        runTime.constant(),
        mesh,
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    )
);

// read a few switches and some data
Switch meshMotion(dynamicMeshDict.lookup("meshMotion"));
dimensionedVector v0(dynamicMeshDict.lookup("v0"));
dimensionedScalar fluxFactorThreshold(dynamicMeshDict.lookup("fluxFactorThreshold"));
scalar fluxThreshold = fluxFactorThreshold.value();
word movingPatchName(dynamicMeshDict.lookup("movingPatch"));
word exitPatchName(dynamicMeshDict.lookup("exitPatch"));
vector pointMotionDirection(dynamicMeshDict.lookup("pointMotionDirection"));
Switch developper(dynamicMeshDict.lookup("developper"));

// Policing option compatibilities
if (volAblation && !finiteRate && !developper)
{
    Info << "WARNING: Finite-rate volume ablation (volAblation) is not compatible with equilibrium chemistry. Exiting PAM." << nl;
    return (0);
}

// Policing option compatibilities
if (!mutationTT && finiteRate && !developper)
{
    Info << "WARNING: Finite-rate volume ablation needs to be run with mutationTT. We cannot rely on tables. Exiting PAM." << nl;
    return (0);
}

// prepare the structure for the elastic motion solver

volVectorField Motionw
(
    IOobject
    (
        "Motionw",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    v0
);

label movingPatchID = mesh.boundaryMesh().findPatchID(movingPatchName);
label exitPatchID   = mesh.boundaryMesh().findPatchID(exitPatchName);

if (movingPatchID < 0)
{
    FatalErrorIn(args.executable())
        << "The moving patch name has not been found. "
            "It needs to be defined - even if the motion is set to zero. "
            "Please check input file 'dynamicMeshDict'."
        << abort(FatalError);
}

// Grab motion field
// Read moving patch data
word movingRegionName(dynamicMeshDict.lookup("movingRegion"));

const fvMesh &motionMesh =
    runTime.objectRegistry::lookupObject<fvMesh>(movingRegionName);

volVectorField cellMotionU
(
    IOobject
    (
        "cellMotionU",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    ),
    mesh
);

volVectorField& motionU =
    const_cast<volVectorField&>
    (
        motionMesh.objectRegistry::lookupObject<volVectorField>
        (
            "cellMotionU"
        )
    );

scalarField tau_w = tau.boundaryField()[movingPatchID];

if (!convectiveBoundaryLayer && readBoundaryTecplotTable)
{
    Info << "boundaryLayer has to be true with readBoundaryTecplotTable = true." << endl;
    return (0);
}
