IOdictionary setCase
(
    IOobject
    (
        "simulationOptions/setCase",
        runTime.constant(),
        runTime.db(),
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    )
);

/*** Boundary condition setup ***/
scalar failureFraction = setCase.lookupOrDefault<scalar>("failureFraction", 0.0);
int physicsBasedErosionModel = setCase.lookupOrDefault<int>("physicsBasedErosionModel", 0);
Switch volAblation(setCase.lookup("volAblation"));
Switch constrainedEquilibrium = setCase.lookupOrDefault<Switch>("constrainedEquilibrium", "no");
scalar erosionRate = setCase.lookupOrDefault<scalar>("erosionRate", 0.0);
scalar wallSpeciesDiffusion = setCase.lookupOrDefault<scalar>("wallSpeciesDiffusion", 1);
Switch mutationBprime(setCase.lookup("mutationBprime"));
// Options for axisymmetrical fluxes
Switch axiSymmetricalFlux(setCase.lookup("axiSymmetricalFlux"));
vector fluxFactorNormal(setCase.lookup("fluxFactorNormal"));
vector fluxFactorCenter(setCase.lookup("fluxFactorCenter"));
Switch fluxFactorProjection(setCase.lookup("fluxFactorProjection"));

word topBoundaryCondition = setCase.lookupOrDefault<word>("topBoundaryCondition","none");
Switch cfd (false);
Switch origin (true);
Switch convectiveBoundaryLayer(false);

if (topBoundaryCondition=="imposedTpv")
{
//default
}
else if (topBoundaryCondition=="convectiveBoundaryLayer")
{ 
convectiveBoundaryLayer = true;
}
else if (topBoundaryCondition=="flowCoupled")
{
origin = false;
cfd = true;
}
else if (topBoundaryCondition=="none")
{
cfd = true;
}
else
{
Info << "Unknown topBoundaryCondition" << nl;
return(0);
}
// Setting-up (old) code switches no longer seen by users (we need to progressively remove them, jl 19 oct 2017)



/*** In-depth model setup ***/
Switch detailedSolidEnthalpies = setCase.lookupOrDefault<Switch>("detailedSolidEnthalpies", "no");
Switch verifyThermalEquilibrium = setCase.lookupOrDefault<Switch>("verifyThermalEquilibrium", "no");
word chemistryTransport = setCase.lookupOrDefault<word>("chemistryTransport", "none");
word diffusionModel = setCase.lookupOrDefault<word>("diffusionModel","none");
word energyConservation = setCase.lookupOrDefault<word>("energyConservation","none");

// Setting-up (old) code switches no longer seen by users (we need to progressively remove them, jl 19 oct 2017)
Switch porous(true); // type 2
Switch finiteRate(false); // type 2
Switch elementConservation(false); //type 2
Switch mutationTT(false); // type 2

if (chemistryTransport=="nonPorous")
{
porous = false;
}
else if (chemistryTransport=="tabulatedGasProperties")
{ 
//default type 2 model
}
else if (chemistryTransport=="equilibriumConstantElementalComposition")
{   
mutationTT = true;
}
else if (chemistryTransport=="equilibriumElementConservation")
{
mutationTT = true;
elementConservation = true;
}
else if (chemistryTransport=="finiteRateSpeciesConservation")
{
mutationTT = true;
finiteRate = true;
}
else
{
Info << "Unknown chemistryTransport model" << nl;
return(0);
}

Switch multiComponent(false);
if(diffusionModel == "multicomponent") {multiComponent = true;}
else if(diffusionModel == "binary") {multiComponent = false;}
else if(diffusionModel == "none") {multiComponent = false;}
else
{
Info << "Unknown diffusion model" << nl;
return(0);
}


