// set the boundary conditions on the porous material surface in the case of non coupled simulations
if (!cfd)
{
    #include "setPressureBC.H" 	// surface pressure
    #include "setMotion.H"          // surface motion
if (energyConservation != "isothermal")
{
    #include "setEnergyBC.H"   	// surface temperature
}
}
else
{
    #include "setMotion.H"          // surface motion
}

// compute pyrolysis rates
Info << "Advancing pyrolysis" << nl;
#include "pyrolysis.H"

// update properties
Info << "Updating derived properties" << nl;
Tg=Ta; // One temperature model
#include "updateMaterialProperties.H"
#include "updateGasProperties.H"

// solve chemistry and transport
Info << "Chemistry and species/elements conservation" << nl;

if (chemistryTransport=="nonPorous")
{
    #include "updateTabulatedGasProperties.H"
}
else if (chemistryTransport=="tabulatedGasProperties")
{ 
    #include "gMass.H"
}
else if (chemistryTransport=="equilibriumConstantElementalComposition")
{   
    #include "gMass.H"
}
else if (chemistryTransport=="equilibriumElementConservation")
{
    #include "gMass.H"
    #include "setElementsBCgrad.H" // update wall element composition (from B')
    #include "ZEqn.H" // solve element conservation (in equilibrium mode only)
}
else if (chemistryTransport=="finiteRateSpeciesConservation")
{
    #include "gMass.H"
    if (volAblation)
    {
        #include "setSpeciesBC.H" // set boundary layer species concentrations at the external boundary
    }
    if(speciesConservation == "localEuler")
    {
        #include "YEqn_localEuler.H" // solve gas species mass-fraction conservation equation
    }
    else
    {
        #include "YEqn_Euler.H" // solve gas species mass-fraction conservation equation
    }
    if (volAblation)
    {
        #include "sMass.H" // solve solid-mass conservation equation (in-depth ablation)
    }
}
else
{
Info << "Unknown chemistryTransport model" << nl;
return(0);
}

#include "updateGasProperties.H"


// solve energy conservation equations
if (energyConservation != "isothermal")
{
Info << "Energy conservation" << nl;
#include "energy2T.H"           // porous material
#include "energySubStructure.H" // substructure materials
}

// output solid data files
Info << "Writing data" << nl;
#include "write.H"

