/*---------------------------------------------------------------------------*\
YEqn.H

Description
    Species conservation equation (finite-rate chemistry mode).
\*---------------------------------------------------------------------------*/

epsphi = mDotGface & mesh.Sf();           // Mass flux of pyrolysis gases.
epsrhog = eps_g * rho_g;                  // Volume-averaged gas density.

    scalarField Tgi = T.internalField();
    scalarField pgi = p.internalField();
    scalarField rhogi = rho_g.internalField();

tmp<fv::convectionScheme<scalar> > mvConvection
(
    fv::convectionScheme<scalar>::New
    (
        mesh,
        fields,
        epsphi,
        mesh.divScheme("div(epsphi,Yi)")
    )
);

if (volAblation)
{
    Y[iCs] = specificSurface * siteDensity * M_Cs / rho_g ; // loads apparent mass fraction of porous C(gr)
        // is divided by "M_Cs / rho_g" in chemistry solver to get the concentration
}

// computing rates and updating dtChem
dtChem = chemistry.solve
(
    dtChem,
    Tgi,
    pgi,
    rhogi
);

if (volAblation)
{
    forAll(omegaHeterogeneousTotal, i)
    {
    omegaHeterogeneousTotal[i] = - chemistry.RR(iCs)[i]; // store heterogeneous reaction rates for sMass.H
    }

    Y[iCs] = 0.0 * Y[iCs]; // reset solid fraction to zero 
}

// label inertIndex = -1;
volScalarField Yt = 0.0 * Y[0];
for (label i = 0; i < Y.size(); i++)
    {
        if (i != iCs)  //  Solid Carbon conservation is done in sMass.H (not transported)
        {
            volScalarField& Yi = Y[i];                  // Species i mass fraction field
            volScalarField& Dmi = Dm[i];                // Species i average multicomponent diffusion coefficient field     Yini = Y[i].oldtime();
            volScalarField totalYpi = piTotal;          // Species i production rate by the pyrolysis reactions
            totalYpi = totalYpi * 0;

            forAll(Xsi, j)
            {
                totalYpi += Yp[j][i] * pi[j];
            }

            fvScalarMatrix YiEqn
            (
                  fvm::ddt(epsrhog, Yi)                              // storage
                - fvm::div(fvc::interpolate(epsrhog)*mesh.phi(), Yi) // mesh motion correction (ALE)
                + mvConvection->fvmDiv(epsphi, Yi)                   // convection
                - fvm::laplacian(Dmi / eta0 * epsrhog, Yi)           // fickian diffusion
                - totalYpi                                           // species produced by the pyrolysis reactions
                ==
                  eps_g * chemistry.RR(i)			     // finite-rate chemistry production (computed in 'chemistry.H')
		+ fvOptions(epsrhog, Yi)
            );

           // YiEqn.relax();
           // fvOptions.constrain(YiEqn);
            YiEqn.solve(mesh.solver("Yi"));
           // fvOptions.correct(Yi);

            Yi.max(0.0);
            Yt += Yi;
        }
    }

    for (label i = 0; i < Y.size(); i++)
    {
        Y[i] = Y[i] / Yt;
    }

/*** Compute new delatT  ***/

// set tolerance on mass fraction variation rate
scalar Ythreshold = 1e-3;
if (runTime.controlDict().found("Ythreshold"))
{
    Ythreshold = readScalar(runTime.controlDict().lookup("Ythreshold"));
}

scalar dYtolMin = 0.01;
if (runTime.controlDict().found("dYtolMin"))
{
    dYtolMin = readScalar(runTime.controlDict().lookup("dYtolMin"));
}

scalar dYtolMax = 0.05;
if (runTime.controlDict().found("dYtolMax"))
{
    dYtolMax = readScalar(runTime.controlDict().lookup("dYtolMax"));
}

volScalarField Yini = 0.0 * Y[0];
scalar dYmax = 0;
scalar speciesMax = 0;
scalar cellMax = 0;

for (label i = 0; i < Y.size(); i++)
{
      if (i != iCs)
      {
        Yini = Y[i].oldTime();
        forAll(Yini, celli)
        {
            if (Yini[celli] > Ythreshold)
            {
                if (fabs((Yini[celli] - Y[i][celli]) / Yini[celli])  > dYmax)
                {
                    dYmax = fabs((Yini[celli] - Y[i][celli]) / Yini[celli]);
                    speciesMax = i; cellMax = celli;
                }
            }
        }
    }
}

Info << "dYmax(species: " << Y[speciesMax].name()
     << "; in cell: " << cellMax
     << ") = " << dYmax
     << " | Y = " << Y[speciesMax][cellMax]
     << nl;

if (dYmax > dYtolMax)
{
    dtChemYEqn = runTime.deltaT().value() / 1.2;
}

if (dYmax < dYtolMin)
{
    dtChemYEqn = runTime.deltaT().value() * 1.2;
}

// retain the min chemical time step bewteen chemistry (dtChem) and transport (dtChemYEqn)
if (chemTransEulerStepLimiter)
{
 dtChemYEqn = min(dtChemYEqn, dtChem);
}

