/*---------------------------------------------------------------------------*\
YEqn.H

Description
    Species conservation equation (finite-rate chemistry mode).
\*---------------------------------------------------------------------------*/

epsphi = mDotGface & mesh.Sf();           // Mass flux of pyrolysis gases.
epsrhog = eps_g * rho_g;                  // Volume-averaged gas density.

    scalarField Tgi = T.internalField();
    scalarField pgi = p.internalField();
    scalarField rhogi = rho_g.internalField();

tmp<fv::convectionScheme<scalar> > mvConvection
(
    fv::convectionScheme<scalar>::New
    (
        mesh,
        fields,
        epsphi,
        mesh.divScheme("div(epsphi,Yi)")
    )
);


int iterChemTrans = 0;
scalar tChemTrans = 0;
scalar dtChem_ =0;

if (volAblation)
{
    omegaHeterogeneousTotal = 0.0 * omegaHeterogeneousTotal;
}

while(tChemTrans < runTime.deltaT().value())
{
dtChem_ = min (dtChem, runTime.deltaT().value());
dtChem_ = min (dtChem_, runTime.deltaT().value()-tChemTrans);


    if(LTS)
    {
        volScalarField& rDeltaT = trDeltaT.ref();
        forAll(rDeltaT, i)
        { 
            rDeltaT[i] = 1/dtChem_;
        }   
        rDeltaT.correctBoundaryConditions();   
    }

    if (volAblation)
    {
        Y[iCs] = specificSurface * siteDensity * M_Cs / rho_g ; // loads apparent mass fraction of porous C(gr)
        // is divided by "M_Cs / rho_g" in chemistry solver to get the concentration
    }

    dtChem = chemistry.solve
    (
	dtChem_,
        Tgi,
        pgi,
        rhogi
     );

    if (volAblation)
    {
        forAll(omegaHeterogeneousTotal, i)
       {
       omegaHeterogeneousTotal[i] -= chemistry.RR(iCs)[i] * dtChem_ ;
       }

        Y[iCs] = 0.0 * Y[iCs]; // reset effective solid fraction to zero 
    }

    // label inertIndex = -1;
    volScalarField Yt = 0.0 * Y[0];
    for (label i = 0; i < Y.size(); i++)
    {
        if (i != iCs)  //  Solid Carbon conservation is done in sMass.H (not transported)
        {
            volScalarField& Yi = Y[i];                  // Species i mass fraction field
            volScalarField& Dmi = Dm[i];                // Species i average multicomponent diffusion coefficient field     Yini = Y[i].oldtime();
            volScalarField totalYpi = piTotal;          // Species i production rate by the pyrolysis reactions
            totalYpi = totalYpi * 0;

            forAll(Xsi, j)
            {
                totalYpi += Yp[j][i] * pi[j];
            }

            fvScalarMatrix YiEqn
            (
                  fvm::ddt(epsrhog, Yi)                              // storage
                - fvm::div(fvc::interpolate(epsrhog)*mesh.phi(), Yi) // mesh motion correction (ALE) done in mesh.solver if not commented
                + mvConvection->fvmDiv(epsphi, Yi)                   // convection
                - fvm::laplacian(Dmi / eta0 * epsrhog, Yi)           // fickian diffusion
                - totalYpi                                           // species produced by the pyrolysis reactions
                ==
                  eps_g * chemistry.RR(i)			     // finite-rate chemistry production (computed in 'chemistry.H')
		+ fvOptions(epsrhog, Yi)
            );

           // YiEqn.relax();
           // fvOptions.constrain(YiEqn);
            YiEqn.solve(mesh.solver("Yi"));
           // fvOptions.correct(Yi);

            Yi.max(0.0);
            Yt += Yi;
        }
    }

    for (label i = 0; i < Y.size(); i++)
    {
        Y[i] = Y[i] / Yt;
    }

iterChemTrans += 1;
tChemTrans += dtChem_;
//Info << "Iteration : " << iterChemTrans <<"	Chemical time step : " << dtChem_ << "	Time left : " << runTime.deltaT().value()-tChemTrans << nl ;
}
Info << iterChemTrans << " iteration(s). Mean dtChem = " << runTime.deltaT().value() / iterChemTrans << nl;

//Compute the mean volume ablation rate over dt
if (volAblation)
{
    forAll(omegaHeterogeneousTotal, i)
    {
       omegaHeterogeneousTotal[i] /= runTime.deltaT().value();
    }
}

/* to print the enthalpy of the mixture in J/kg as a function of T (in K)
for (int tvar = 0 ; tvar < 20 ; tvar ++ ){
scalar enth=0;
    for(label i=0; i<Y.size(); i++)
    {
enth = enth + Y[i][0] * chemistry.hsi(i, 200 + tvar *100);
    }
Info << "h[T= " << 200 + tvar * 100 << "] =" << enth << nl;
}
*/

/*** Compute new delatT  ***/

// set tolerance on mass fraction variation rate
scalar Ythreshold = 1e-3;
if (runTime.controlDict().found("Ythreshold"))
{
    Ythreshold = readScalar(runTime.controlDict().lookup("Ythreshold"));
}

scalar dYtolMin = 0.01;
if (runTime.controlDict().found("dYtolMin"))
{
    dYtolMin = readScalar(runTime.controlDict().lookup("dYtolMin"));
}

scalar dYtolMax = 0.05;
if (runTime.controlDict().found("dYtolMax"))
{
    dYtolMax = readScalar(runTime.controlDict().lookup("dYtolMax"));
}

volScalarField Yini = 0.0 * Y[0];
scalar dYmax = 0;
scalar speciesMax = 0;
scalar cellMax = 0;

for (label i = 0; i < Y.size(); i++)
{
      if (i != iCs)
      {
        Yini = Y[i].oldTime();
        forAll(Yini, celli)
        {
            if (Yini[celli] > Ythreshold)
            {
                if (fabs((Yini[celli] - Y[i][celli]) / Yini[celli])  > dYmax)
                {
                    dYmax = fabs((Yini[celli] - Y[i][celli]) / Yini[celli]);
                    speciesMax = i; cellMax = celli;
                }
            }
        }
    }
}

Info << "dYmax(species: " << Y[speciesMax].name()
     << "; in cell: " << cellMax
     << ") = " << dYmax
     << " | Y = " << Y[speciesMax][cellMax]
     << nl;

if (dYmax > dYtolMax)
{
    dtChemYEqn = runTime.deltaT().value() / 1.2;
}

if (dYmax < dYtolMin)
{
    dtChemYEqn = runTime.deltaT().value() * 1.2;
}


