/*---------------------------------------------------------------------------*\
ZEqn.H

Description
    Element conservation equation (equilibrium chemistry mode).
\*---------------------------------------------------------------------------*/

epsphi  = mDotGface & mesh.Sf(); // Mass flux of pyrolysis gases.
epsrhog = eps_g * rho_g;

tmp<fv::convectionScheme<scalar> > mvConvection
(
    fv::convectionScheme<scalar>::New
    (
        mesh,
        fields,
        epsphi,
        mesh.divScheme("div(epsphi,Zi)")
    )
);


{
    volScalarField Zt = 0.0 * Z[0];
    ZtotMassFlux = 0.0 * ZmassFlux[0];

    for (label i = 0; i < Z.size(); i++)
    {
        volScalarField& Zi = Z[i];
        volScalarField& Dmi = Dm[i];
        volScalarField totalZp = piTotal;
        totalZp = 0. * totalZp;
        forAll(Xsi, j)
        {
            totalZp += Zp[j][i] * pi[j];
        }

        fvScalarMatrix ZiEqn
        (
              fvm::ddt(epsrhog, Zi)                    // storage
            - fvm::div(fvc::interpolate(epsrhog)*mesh.phi(), Zi)  // mesh motion correction (ALE)
            + mvConvection->fvmDiv(epsphi, Zi)         // convection
            - fvm::laplacian(Dmi / eta0 * epsrhog, Zi) // diffusion
            - totalZp                 // elements produced by the pyrolysis reactions
            == fvOptions(epsrhog, Zi)
        );

        // ZiEqn.relax();
        // fvOptions.constrain(ZiEqn);
        ZiEqn.solve(mesh.solver("Zi"));
        // fvOptions.correct(Zi);

        Zi.max(0.0);
        Zt += Zi;

        // compute the mass diffusion fluxes of the elements
        // convection - implement as zeroGradient, ie, we need the sub surface cell value at the boundary.
        ZmassFlux[i] =  -  fvc::interpolate(Dmi / eta0 * epsrhog) * (mesh.Sf() / mesh.magSf()) * fvc::snGrad(Zi); //diffusion
        ZtotMassFlux += ZmassFlux[i]; // average mass diffusion flux
    }

    for (label i = 0; i < Z.size(); i++)
        Z[i] /= Zt;
} 

