// Gas thermodynamics and transport properties

// Update gas temperature (lagged in the energy conservation)
Tg = Ta; // One temperature model
T = Tg;  // T is looked-up by the chemistry solver

scalar T_var = BL.Tinf(runTime.value());
scalar p_var = BL.pw(runTime.value());

if (!readBoundaryTecplotTable)  // to be discussed with Jeremy Meurisse - jl, 19 Oct 2017
{
    pTp[1] = T_var; pTp[0] = p_var;
    mix.setState(p_Zx, pTp, 2);
}

forAll (M, i)
{
if (!readBoundaryTecplotTable)
    {
        if 
        ((T_var > Tg[i] + 10.) ||
        (T_var < Tg[i] - 10.) ||
        (p_var > min((p[i] +  p[i] / 10.), (p[i] + 1000.))) ||
        (p_var < max((p[i] - p[i] / 10.), (p[i] - 1000.)))
        )
        {
        T_var  = Tg[i];
        p_var  = p[i];
        pTp[1] = T_var;
        pTp[0] = p_var;
        mix.setState(p_Zx, pTp, 2);
        }
        // else do nothing 
     }
else
{
    T_var  = Tg[i];
    p_var  = p[i];
    pTp[1] = T_var;
    pTp[0] = p_var;
    mix.setState(p_Zx, pTp, 2);
}
                
M[i]  = mix.mixtureMw();
hg[i] = mix.mixtureHMass();
mu[i] = mix.viscosity();
}

forAll(M.boundaryField()[exitPatchID], faceI)
{
    if (!readBoundaryTecplotTable)
    {
         if
         ((T_var > Tg.boundaryField()[exitPatchID][faceI] + 10.) ||
         (T_var < Tg.boundaryField()[exitPatchID][faceI] - 10.) ||
         (p_var > p.boundaryField()[exitPatchID][faceI] + 10.) ||
         (p_var < p.boundaryField()[exitPatchID][faceI] - 10.)
         )
         {
             T_var  = Tg.boundaryField()[exitPatchID][faceI];
             p_var  = p.boundaryField()[exitPatchID][faceI];
             pTp[1] = T_var;
             pTp[0] = p_var;
             mix.setState(p_Zx, pTp, 2);
          }
     }
     else
     {
     T_var  = Tg.boundaryField()[exitPatchID][faceI];
     p_var  = p.boundaryField()[exitPatchID][faceI];
     pTp[1] = T_var;
     pTp[0] = p_var;
     mix.setState(p_Zx, pTp, 2);
     }
M.boundaryFieldRef()[exitPatchID][faceI]  = mix.mixtureMw();
hg.boundaryFieldRef()[exitPatchID][faceI] = mix.mixtureHMass();
mu.boundaryFieldRef()[exitPatchID][faceI] = mix.viscosity();
}

forAll(M.boundaryField()[movingPatchID], faceI)
{
    if (!readBoundaryTecplotTable)
    {
    if
    ((T_var > Tg.boundaryField()[movingPatchID][faceI] + 10.) ||
    (T_var < Tg.boundaryField()[movingPatchID][faceI] - 10.) ||
    (p_var > p.boundaryField()[movingPatchID][faceI] + 10.) ||
    (p_var < p.boundaryField()[movingPatchID][faceI] - 10.)
    )
    {
    T_var  = Tg.boundaryField()[movingPatchID][faceI];
    p_var  = p.boundaryField()[movingPatchID][faceI];
    pTp[1] = T_var;
    pTp[0] = p_var;
    mix.setState(p_Zx, pTp, 2);
    }
    }
else
    {
    T_var  = Tg.boundaryField()[movingPatchID][faceI];
    p_var  = p.boundaryField()[movingPatchID][faceI];
    pTp[1] = T_var;
    pTp[0] = p_var;
    mix.setState(p_Zx, pTp, 2);
    }

M.boundaryFieldRef()[movingPatchID][faceI]  = mix.mixtureMw();
hg.boundaryFieldRef()[movingPatchID][faceI] = mix.mixtureHMass();
mu.boundaryFieldRef()[movingPatchID][faceI] = mix.viscosity();
}

// Densities
rho_g  = p * M / (R * Tg); // gas density (perfect gas law)
rho[0] = rho_g;

// Notes : 
// 1- multicomponent diffusion of elements is not implemented (in development in Mutation++)
// 2- element enthalpy diffusion is not implemented (in development in Mutation++)
// As a consequence the model is correct when element diffusion is neglected.
