// Gas thermodynamics and transport properties

// Update gas temperature (lagged in the energy conservation)
Tg = Ta; // One temperature model
T = Tg;  // T is looked-up by the chemistry solver

            forAll (M, cellI)
            {
                pTp[1] = Tg[cellI];
                pTp[0] = p[cellI];

                for (int i = 0; i < ne_mix; i++)
                {
                    p_Zx[i] = Z[i][cellI];
                }

                mix.convert<Mutation::Thermodynamics::YE_TO_XE>(p_Zx, p_Zx);
                mix.setState(p_Zx, pTp, 2);
                M[cellI]  = mix.mixtureMw();
                hg[cellI] = mix.mixtureHMass();
                mu[cellI] = mix.viscosity();
            }

            forAll(M.boundaryField()[exitPatchID], faceI)
            {
                pTp[1] = Tg.boundaryField()[exitPatchID][faceI];
                pTp[0] = p.boundaryField()[exitPatchID][faceI];

                for (int i = 0; i < ne_mix; i++)
                {
                    p_Zx[i] = Z[i].boundaryField()[exitPatchID][faceI];
                }
                mix.convert<Mutation::Thermodynamics::YE_TO_XE>(p_Zx, p_Zx);
                mix.setState(p_Zx, pTp, 2);
                M.boundaryFieldRef()[exitPatchID][faceI]  = mix.mixtureMw();
                hg.boundaryFieldRef()[exitPatchID][faceI] = mix.mixtureHMass();
                mu.boundaryFieldRef()[exitPatchID][faceI] = mix.viscosity();
            }

            forAll(M.boundaryField()[movingPatchID], faceI)
            {
                pTp[1] = Tg.boundaryField()[movingPatchID][faceI];
                pTp[0] = p.boundaryField()[movingPatchID][faceI];

                for (int i = 0; i < ne_mix; i++)
                {
                    p_Zx[i] = Z[i].boundaryField()[movingPatchID][faceI];
                }
                mix.convert<Mutation::Thermodynamics::YE_TO_XE>(p_Zx, p_Zx);
                mix.setState(p_Zx, pTp, 2);
                M.boundaryFieldRef()[movingPatchID][faceI]  = mix.mixtureMw();
                hg.boundaryFieldRef()[movingPatchID][faceI] = mix.mixtureHMass();
                mu.boundaryFieldRef()[movingPatchID][faceI] = mix.viscosity();
            }

// Densities
rho_g  = p * M / (R * Tg); // gas density (perfect gas law)
rho[0] = rho_g;

// Notes : 
// 1- multicomponent diffusion of elements is not implemented (in development in Mutation++)
// 2- element enthalpy diffusion is not implemented (in development in Mutation++)
// As a consequence the model is correct when element diffusion is neglected.
