// Gas thermodynamics and transport properties

// Update gas temperature
T = Tg;  // T is looked-up by the chemistry solver

    forAll (M, cellI)
    {
        // initialisation of the pointers for T,p (pTp) and for the species (pY), used in setState(pY,pTp,2) in Mutation++
        pTp[1] = Tg[cellI];
        pTp[0] = p[cellI];

        for (int i = 0; i < ns; i++)
        {
            pY[speciesIndexMutation[i]] = Y[i][cellI];
        }

        // set mixture composition in mutation++
        mixFiniteRate.setState(pY, pTp, 2);

        // compute thermochemical data
        M[cellI]  = mixFiniteRate.mixtureMw();
        hg[cellI] = mixFiniteRate.mixtureHMass();
        mu[cellI] = mixFiniteRate.viscosity();

        if (diffusionModel == "multicomponent")
        {
            mixFiniteRate.averageDiffusionCoeffs(Ds);
            for (int i = 0; i < ns; i++)
            {
                Dm[i][cellI] = Ds[speciesIndexMutation[i]];
            }
        }

        mixFiniteRate.speciesHOverRT(Tg[cellI], hSpecies);
        for (int i = 0; i < ns; i++)
        {
            hSpeciesField[i][cellI] =
                hSpecies[speciesIndexMutation[i]] / mixFiniteRate.speciesMw(speciesIndexMutation[i]) * R.value() * Tg[cellI];
        }
    }

    forAll (M.boundaryField()[movingPatchID], faceI)
    {
        // initialisation of p,T (pTp) and species (pY) pointers for setState(pY,pTp,2) of mutation++
        pTp[1] = Tg.boundaryField()[movingPatchID][faceI];
        pTp[0] = p.boundaryField()[movingPatchID][faceI];

        for (int i = 0; i < ns; i++)
        {
            pY[speciesIndexMutation[i]] = Y[i].boundaryField()[movingPatchID][faceI];
        }

        mixFiniteRate.setState(pY, pTp, 2);
        M.boundaryFieldRef()[movingPatchID][faceI]  = mixFiniteRate.mixtureMw();
        hg.boundaryFieldRef()[movingPatchID][faceI] = mixFiniteRate.mixtureHMass();
        mu.boundaryFieldRef()[movingPatchID][faceI] = mixFiniteRate.viscosity();

        if (diffusionModel == "multicomponent")
        {
            mixFiniteRate.averageDiffusionCoeffs(Ds);
            for (int i = 0; i < ns; i++)
            {
                Dm[i].boundaryFieldRef()[movingPatchID][faceI] = Ds[speciesIndexMutation[i]];
            }
        }

        mixFiniteRate.speciesHOverRT(Tg.boundaryField()[movingPatchID][faceI], hSpecies);
        for (int i = 0; i < ns; i++)
        {
            hSpeciesField[i].boundaryFieldRef()[movingPatchID][faceI] =
                hSpecies[speciesIndexMutation[i]] / mixFiniteRate.speciesMw(speciesIndexMutation[i]) *
                R.value() *
                Tg.boundaryField()[movingPatchID][faceI];
        }
    }

// Densities
rho_g  = p * M / (R * Tg); // gas density (perfect gas law)
rho[0] = rho_g;

// compute the energy (flux) transported by diffusion of the molecules
Ediff = Ediff0;
for (label i = 0; i < Y.size(); i++)
{
    if (i != iCs)
    {
    volScalarField& Dmi = Dm[i];
    volScalarField& hSpeciesFieldi = hSpeciesField[i];

    // energy (enthalpy) transported by molecular diffusion
    Ediff = Ediff + fvc::laplacian(Dmi / eta0 * eps_g * rho_g * hSpeciesFieldi, Y[i]);
    }
}

   // and update on movingPatch 

    scalarField invDxw    = mesh.deltaCoeffs().boundaryField()[movingPatchID];
    scalarField epsgw     = eps_g.boundaryField()[movingPatchID];
    scalarField rhogw     = rho[0].boundaryField()[movingPatchID];
    hwDiff.boundaryFieldRef()[movingPatchID] = hwDiff.boundaryField()[movingPatchID] * 0;

    for (int j = 0; j < Y.size(); j++)
        {
         if (j != iCs)  //  Solid Carbon doesn't diffuse
             {
    scalarField Yiw = Y[j].boundaryField()[movingPatchID];
    scalarField Yis = Y[j].boundaryField()[movingPatchID].patchInternalField();
    scalarField Dmiw = Dm[j].boundaryField()[movingPatchID].patchInternalField();
    scalarField hSpeciesFieldiw = hSpeciesField[j].boundaryField()[movingPatchID].patchInternalField();

              hwDiff.boundaryFieldRef()[movingPatchID] 
              += Dmiw * invDxw * epsgw * rhogw * hSpeciesFieldiw / eta0.value() * (Yiw-Yis);

             }
         }

// enthalpy of the solid lost with heterogeneous reactions
omegaHeterogeneousEnergy = - h_c * omegaHeterogeneousTotal;
