/*---------------------------------------------------------------------------*
setTimeStep_gradp.H

An adaptative time step is set on the max pressure gradient. This is a predictor of the maximum time step that will give time-accurate results for the mass conservation equation. Since the mass consevation equation integrates the Darcy law, it is important to control that the flux transported in each cell (proportional to the pressure gradient) is small enough compared to other quantities. This is not a CFL as the scheme used is unconditionnaly stable. It is only a predictor for time accuracy that can be turned off (keyword 'adjustTimeStep' in system/controlDict) and replaced by the user by a standard time accuracy study.

\*---------------------------------------------------------------------------*/

/**** Initialization ****/

Switch adjustTimeStep
(
    runTime.controlDict().lookup("adjustTimeStep")
);

scalar maxCo
(
    readScalar(runTime.controlDict().lookup("maxCo"))
);

scalar maxDeltaT = GREAT;

if (runTime.controlDict().found("maxDeltaT"))
{
    maxDeltaT = readScalar(runTime.controlDict().lookup("maxDeltaT"));
}

scalar REVlength = 1e-3;

if (runTime.controlDict().found("REVlength"))
{
    REVlength = readScalar(runTime.controlDict().lookup("REVlength"));
}

scalar minChemDeltaT = SMALL;

if (runTime.controlDict().found("minChemDeltaT"))
{
    minChemDeltaT = readScalar(runTime.controlDict().lookup("minChemDeltaT"));
}

scalar minDeltaT = SMALL;

if (runTime.controlDict().found("minDeltaT"))
{
    minDeltaT = readScalar(runTime.controlDict().lookup("minDeltaT"));
}

scalar adjustStartTime
(
    readScalar(runTime.controlDict().lookup("adjustStartTime"))
);

/**** Set "CFL" number ****/

scalar CoNum = 0.0;
// scalar meanCoNum = 0.0;
// scalar velMag = 0.0;
vS = linearInterpolate(v);

if (mesh.nInternalFaces())
{
    surfaceScalarField SfUfbyDelta =
        mesh.surfaceInterpolation::deltaCoeffs() * mag(vS) / (REVlength);

    CoNum = max(SfUfbyDelta).value() * runTime.deltaT().value();

    // meanCoNum =
    //     (
    //         sum(SfUfbyDelta) /
    //         sum(mesh.magSf())
    //     ).value() *
    //     runTime.deltaT().value();

    // velMag = max(mag(vS)).value();

    /*
        CoNum = max(SfUfbyDelta/mesh.magSf())
            .value()*runTime.deltaT().value();

        meanCoNum = (sum(SfUfbyDelta)/sum(mesh.magSf()))
            .value()*runTime.deltaT().value();

        velMag = max(mag(vS)/mesh.magSf()).value();
    */
}
/*
Info<< "Courant Number mean: " << meanCoNum
    << " max: " << CoNum
    << " velocity magnitude: " << velMag << endl;
*/
// ***** Adjust time step ***** //

if (adjustTimeStep && (runTime.value() > adjustStartTime))
{
    scalar maxDeltaTFact = maxCo / (CoNum + SMALL);
    scalar deltaTFact = min(min(maxDeltaTFact, 1.0 + 0.2 * maxDeltaTFact), 1.05);
    // scalar deltaTFact = min(min(maxDeltaTFact, 1.0 + 0.5*maxDeltaTFact), 10);

    // Info << "maxDeltaTFact initial = " << maxDeltaTFact << nl;
    // Info << "deltaTFact damped = " << deltaTFact << nl;

    scalar deltaTphys =
        max
        (
            min
            (
                deltaTFact * runTime.deltaT().value(),
                maxDeltaT
            ),
            minDeltaT
        );

    if (finiteRate)
    {
        runTime.setDeltaT(max(min(dtChemYEqn, deltaTphys), minChemDeltaT));
    }
    else
    {
        runTime.setDeltaT(deltaTphys);
    }
}
