/*---------------------------------------------------------------------------*\
Beta version (development) of a simple pyrolysis solver using PAM pyrolysis alogorythm
\*---------------------------------------------------------------------------*/

// Main library openFOAM
#include "fvCFD.H"

// dynamicMesh libraries
#include "dynamicFvMesh.H"
#include "dynamicMotionSolverFvMesh.H"

// stream libraries
#include "IFstream.H"
#include "IOstream.H"
#include "ISstream.H"
#include "OFstream.H"

// boundary condition libraries
#include "fixedValueFvPatchField.H"
#include "fixedGradientFvPatchField.H"
#include "fixedGradientFvPatchFields.H"

// probes libraries
#include "probes.H"
#include "IOprobes.H"

// Mutation++ library
#undef Log // conflict between OpenFoam and Mutation++ on the alias "Log" -> undefined here from OpenFoam.
#include <mutation++/mutation++.h>
#include <sys/stat.h>
#include <sys/types.h>

// options library from openFOAM
#include "fvOptions.H"

// regions library
#include "regionProperties.H"

// chemistry library
#include "psiReactionThermo.H"
#include "psiChemistryModel.H"
#include "chemistrySolver.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
  // set root case and time
#include "setRootCase.H"
#include "createTime.H"

  // region (porousMat,subMat,...)
  regionProperties rp(runTime);

  // set the meshes and read properties
#include "createFields/createSolidMeshes.H"
#include "createFields/createPorousFields.H"
#include "createFvOptions.H"
#include "input/readMotionProperties.H"
#include "createFields/createStreams.H"
#include "input/readChemistryProperties.H"

  while (runTime.loop())
    {

      Info << "-----------" << nl << "runTime = " << runTime.timeName()
	   << " s ; Time step = " << runTime.deltaT().value() << " s" << endl;

      // Update the dynamic mesh
      mesh.update();

      // Update temperature
#include "updateDerivedFields/updateTemperature.H"

      // Compute the pyrolysis rate
      Info << "Pyrolysis" << nl;
#include "chemistry/pyrolysis.H"

      // Update solid density after pyrolysis
#include "updateDerivedFields/updateRhoPi.H"

      Info << "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
	   << "  ClockTime = " << runTime.elapsedClockTime() << " s"
	   << nl << endl;

    }

  Info << "End\n" << endl;

  return (0);
}


// ************************************************************************* //
