// This is to initialize the dynamic mesh motion -- identification of the moving
// patch (receding surface) and projection of the FV mesh on a tetrahedral mesh
// for FEM resolution of elastic mesh motion.

Info << "Reading Motion Properties" << endl;

// Open the dictionary
IOdictionary dynamicMeshDict
(
    IOobject
    (
        "dynamicMeshDict",
        runTime.constant(),
        mesh,
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    )
);
IOdictionary setCase
(
 IOobject
 (
  "setCase",
  runTime.constant(),
  runTime.db(),
  IOobject::MUST_READ,
  IOobject::NO_WRITE
  )
 );

Switch oneDoutput(dynamicMeshDict.lookup("oneDoutput"));
dimensionedVector v0(dynamicMeshDict.lookup("v0"));
word movingPatchName(dynamicMeshDict.lookup("movingPatch"));

volVectorField Motionw
(
    IOobject
    (
        "Motionw",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    un * v0
);

label movingPatchID = mesh.boundaryMesh().findPatchID(movingPatchName);

if (movingPatchID < 0)
{
    FatalErrorIn(args.executable())
        << "The moving patch name has not been found. "
           "It needs to be defined - even if the motion is set to zero. "
           "Please check input file 'dynamicMeshDict'."
        << abort(FatalError);
}

// Grab motion field
// Read moving patch data
word movingRegionName(dynamicMeshDict.lookup("movingRegion"));

const fvMesh &motionMesh =
    runTime.objectRegistry::lookupObject<fvMesh>(movingRegionName);

volVectorField cellMotionU
(
    IOobject
    (
        "cellMotionU",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::AUTO_WRITE
    ),
    mesh
);

volVectorField& motionU =
    const_cast<volVectorField&>
    (
        motionMesh.objectRegistry::lookupObject<volVectorField>
        (
            "cellMotionU"
        )
    );

